
local MakePlayerCharacter = require "prefabs/player_common"


local assets = {
    Asset("SCRIPT", "scripts/prefabs/player_common.lua"),
}
local prefabs = {}

-- Custom starting items
local start_inv = {
}

-- instakill
local function beckon(inst, target)
	if target ~= nil then
		if target.components.health and target.components.combat then
			if target ~= inst  and not (
					(target:HasTag("player") and not GLOBAL.TheNet:GetPVPEnabled())
					or target:HasTag("wall") 
					or target:HasTag("shadow") 
					or target:HasTag("ghost") 
					or target:HasTag("chess") 
					or target:HasTag("eyeplant") 
					or target.name == "tentacle" 
					or target.name == "tentacle_pillar") then
				local sanityCost = target.components.health.maxhealth * 0.5
				if sanityCost < 50 then sanityCost = 50	end		
				if inst.components.sanity.current > sanityCost then
					inst.components.sanity:DoDelta(-sanityCost, false)
					target.components.health:SetVal(0)
					
					print(type(target).. ", "..target.components.health.currenthealth)
					
					target = nil
					
					return true
				else
					print("Not enough sanity to kill "..target.name)
				end
			else
				print(target.name.." can't be killed due to lack of a soul")
			end
		else
			print("Can't kill "..target.name)
		end
	else
		print("No target")
	end
end

-- When the character is revived from human
local function onbecamehuman(inst)
	-- Set speed when reviving from ghost (optional)
	inst.components.locomotor:SetExternalSpeedMultiplier(inst, "yuyuko_speed_mod", 1)
end

local function onbecameghost(inst)
	-- Remove speed modifier when becoming a ghost
   inst.components.locomotor:RemoveExternalSpeedMultiplier(inst, "yuyuko_speed_mod")
end

-- When loading or spawning the character
local function onload(inst)
    inst:ListenForEvent("ms_respawnedfromghost", onbecamehuman)
    inst:ListenForEvent("ms_becameghost", onbecameghost)

    if inst:HasTag("playerghost") then
        onbecameghost(inst)
    else
        onbecamehuman(inst)
    end
end

local function CalcSanityAura(inst, observer)
	if not observer:HasTag("ghost") then 
		local val = inst.components.sanityaura.aura * ((inst.components.sanity.max - inst.components.sanity.current - 10) / 116)
		if val > 0 then 
			return val
		end
	end
	return 0
end

-- This initializes for both the server and client. Tags can be added here.
local common_postinit = function(inst) 
	-- Minimap icon
	inst.MiniMapEntity:SetIcon( "yuyuko.tex" )
	inst:AddTag("ghost")
	inst:AddTag("yuyuko")
	-- kill target
	inst.beckon = beckon
end

-- This initializes for the server only. Components are added here.
local master_postinit = function(inst)
	-- choose which sounds this character will play
	inst.soundsname = "wendy"
	
	-- Uncomment if "wathgrithr"(Wigfrid) or "webber" voice is used
    --inst.talker_path_override = "dontstarve_DLC001/characters/"
	
	-- Stats	
	inst.components.health:SetMaxHealth(150)
	
	-- hunger
	inst.components.hunger:SetMax(225)
	inst.components.hunger.hungerrate = 1.5 * TUNING.WILSON_HUNGER_RATE
	inst.components.eater:SetCanEatHorrible()
	
	-- sanity
	inst.components.sanity:SetMax(200)
	inst.components.sanity.custom_rate_fn = function(instance)
		return -2.000756 + 0.02644772*instance.components.hunger.current - 0.0001254601*instance.components.hunger.current^2 + 2.187551e-7*instance.components.hunger.current^3
		--((instance.components.hunger.current - (instance.components.hunger.max - 20))^3)/4000000
		--((instance.components.hunger.max - 10) - instance.components.hunger.current) / 200
	end
	---------------- sanity aura
	inst:AddComponent("sanityaura")
    inst.components.sanityaura.aurafn = CalcSanityAura
	inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED
	inst.components.sanity.ghost_drain_mult = 0
	
	-- Damage multiplier (optional)
    inst.components.combat.damagemultiplier = 1
	
	inst.OnLoad = onload
    inst.OnNewSpawn = onload
	
end

return MakePlayerCharacter("yuyuko", prefabs, assets, common_postinit, master_postinit, start_inv)
